import 'dart:async';

import 'package:getx_extended/export.dart';

class RxPageable<T> extends RxList<T> {
  static var pageSize = 20;
  static var defaultFirstPage = 1;

  Future<List<T>> Function(int page, int size) dataSource;

  RxPageable({
    required this.dataSource,
  }) : super() {
    reloadDataSource();
  }

  late int _currentPage;
  late bool _isEnded;
  late bool _isLoading;
  bool _isRefreshing = false;

  Future reloadDataSource() async {
    if (_isRefreshing) return;
    clear();
    _isRefreshing = true;
    _currentPage = defaultFirstPage;
    _isEnded = false;
    _isLoading = false;
    await next();
    _isRefreshing = false;
  }

  T item(int index) {
    final half = pageSize / 2;
    if (index + half > value.length) {
      next();
    }
    return value[index];
  }

  Future next() async {
    if (_isEnded || _isLoading) return;
    _isLoading = true;
    return postFrameFuture(() async {
      try {
        final list = await dataSource(_currentPage, pageSize);
        addAllIf(true, list.toList());
        _currentPage += 1;
        _isEnded = pageSize > list.length;
      } finally {
        _isLoading = false;
      }
    });
  }
}
